@@ -1,5 +1,5 @@
 ActiveRecord::Schema.define do
-  create_table :binary_fields, :force => true do |t|
+  create_table :binary_fields, :force => true, :options => 'CHARACTER SET latin1' do |t|
     t.binary :tiny_blob,   :limit => 255
     t.binary :normal_blob, :limit => 65535
     t.binary :medium_blob, :limit => 16777215
@@ -9,4 +9,4 @@ ActiveRecord::Schema.define do
     t.text   :medium_text, :limit => 16777215
     t.text   :long_text,   :limit => 2147483647
   end
-end
\ No newline at end of file
+end
